/*
 * Copyright(c)  2002  Rijkswaterstaat
 *                     Dienst Verkeerskunde
 *                     WSRA
 * File  :             BUS.INC
 *
 * Versie:       3.2
 *
 * Datum :       7 juni 1999
 * Gewijzigd:    10 maart 2003
 *               Omgezet naar FLASH
 *               TELLERS verwijderd
 *               Bijhouden aantal BRs ipv aantal bussen met prioriteit 
 * Gewijzigd:    26 augustus 2004
 *               aanpassing aan RWS C-versie 7.14,
 *               gebruik EGGPARMS ipv EPARMS
 *               andere afhandeling bussen
 *               loskoppeling busingreep van beperking busingreep
 * gewijzigd:    5 oktober 2005
 *               aanpassing PPB_OP in craptab ivm conflicterende BR
 *
 * Inhoud:
 *
 * In deze file zijn drie functies opgenomen. De functie "bus_ingreep1"
 * (1 rijstrook), "bus_ingreep2" (2 rijstroken) en de functie "beperk_ingreep".
 * De functie bus_ingreep1 (of bus_ingreep2) set een hulpfunctie (bus_op_SG)
 * indien er, op een aangegeven detector, een inmelding van een bus heeft plaats
 * gevonden. De functie beperk_ingreep bewaakt dat er na een n-tal bijzondere
 * realisaties in een cyclus, gedurende de volgende cyclus een beperking van de
 * busingreep plaats zal vinden.
 *
 * Wijzigingen tov de vorige versies zijn ondermeer:
 *  - tellers die voorheen werden gebruikt zijn vervangen door extra geheel getal
 *    parameters;
 *  - er wordt onderscheid gemaakt in het aantal ingemelde bussen en het aantal
 *    te behandelen bussen. De bussen die zich inmelden binnen het zogenaamde
 *    inmeldgebied (tijdens !G en eerste deel G) worden gezien als te behandelen
 *    bussen en krijgen nog de mogelijkheid binnen de huidige groenfase de
 *    stopstreep te passeren. De bussen die te laat in het groen inmelden worden
 *    pas de volgende cyclus afgehandeld.
 *  - tbv het bepalen of er gedurende 1 cyclus een beperkte busingreep moet
 *    plaatsvinden is niet meer afhankelijk van het aantal bussen dat prioriteit
 *    krijgt, maar is afhankelijk van het aantal bijzondere realisaties dat in de
 *    regeling heeft plaatsgevonden. Deze functie is losgekoppeld van de functies
 *    voor de busingrepen. De variabelen in deze functie die niet van belang zijn
 *    voor de programmeur, zijn als interne variabelen gedefinieerd.
 *
 * Let op:
 *  - indien bij 2 rijstroken gebruik wordt gemaakt van 1 in- en uitmeldlus
 *    moet gebruik gemaakt worden van bus_ingreep1
 *  - als de EGGPARM voor het maximum aantal bijzondere realisaties in 1 cyclus
 *    gelijk is aan 0 vindt er geen busbeinvloeding plaats
 */

Bool  br1e;
Int16 blk1e, blkn;

void bus_ingreep1 (Int16 n,               /* SG waar bus op rijdt                                  */
                   Int16 in,              /* inmeldlus                                             */
                   Int16 uit,             /* uitmeldlus                                            */
                   Int16 uitst_bus_SG,    /* tijd uitstel busmelding voor SGn                      */
                   Int16 rijt_bus_SG,     /* ongehinderde rijtijd bus voor SGn                     */
                   Int16 max_groen_SG,    /* maximaal toelaatbare groentijd SGn bij busingreep     */
                   Int16 inm_geb_bus_SG,  /* eparm tbv bepalen inmeldgebied te behandelen bussen   */
                   Int16 aant_bus_SG,     /* aantal ingemelde bussen voor SGn                      */
                   Int16 aant_beh_bus_SG, /* aantal te behandelen bussen voor SGn                  */
                   Int16 bus_op_SG)       /* hulpfunctie te behandelen bus op SGn aanwezig         */
{
/* Inmelding van een bus is niet mogelijk als de uitmeldlus stoort.
   Tevens wordt de uitsteltijd van de busmelding gestart.            */
   if ( begin_D(in) && !DFOUT(uit) )
   {    if ( inst_EGGPARM(aant_bus_SG) == 0 )
        {    start_TIJD(uitst_bus_SG);
        }
        wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) + 1);

/* De minimale rijtijd van de bus wordt gestart bij inmelding
   tijdens groen binnen het inmeldgebied                             */
        if ( !G(n) || G(n) && MG_TIJD(n) && (Int16)waarde_MG_TIJD(n) < inst_EPARM(inm_geb_bus_SG) )
        {    herstart_TIJD(rijt_bus_SG);
             wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_beh_bus_SG) + 1);
        }

/* De maximaal toelaatbare groentijd wordt gestart bij begin groen
   of bij inmelding tijdens groen binnen het inmeldgebied            */
        if ( G(n) && MG_TIJD(n) && (Int16)waarde_MG_TIJD(n) < inst_EPARM(inm_geb_bus_SG) )
        {    herstart_TIJD(max_groen_SG);
        }
   }

   if ( begin_G(n) )
   {    herstart_TIJD(max_groen_SG);
   }

/* Op einde groen wordt het aantal te behandelen bussen gelijk
   gesteld aan het aantal ingemelde bussen                           */
   if ( einde_G(n) )
   {    wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_bus_SG) );
   }

/* Uitmelding van een bus en tellen van het aantal buspassages       */
   if ( begin_D(uit) )
   {    if ( inst_EGGPARM(aant_bus_SG) > 0 )
        {    wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) - 1);
        }
        if ( inst_EGGPARM(aant_beh_bus_SG) > 0 )
        {    wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_beh_bus_SG) - 1);
        }
   }
   
/* Indien de bus voldoende gelegenheid heeft gehad om de stopstreep
   te passeren, wordt de teller die het aantal te behandelen bussen
   aangeeft gereset. De teller van het totaal aantal bussen in het
   traject wordt met het aantal nog te behandelen bussen verlaagd    */
   if (  G(n) && inst_EGGPARM(aant_beh_bus_SG) > 0 && !TIJD(rijt_bus_SG) &&
        (!TIJD(max_groen_SG) || !H1e_TIJD(n) && !H2e_TIJD(n)) )
   {    if ( inst_EGGPARM(aant_bus_SG) >= inst_EGGPARM(aant_beh_bus_SG))
        {    wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) - inst_EGGPARM(aant_beh_bus_SG));
        }
        wijzig_EGGPARM(aant_beh_bus_SG, 0);
   }

/* Indien het aantal bussen 0 is, worden de daaraan
   gerelateerde tijden gereset                                       */
   if ( inst_EGGPARM(aant_bus_SG) == 0 && TIJD(max_groen_SG) )
   {     reset_TIJD(max_groen_SG);
   }
   if ( inst_EGGPARM(aant_bus_SG) == 0 && TIJD(rijt_bus_SG) )
   {     reset_TIJD(rijt_bus_SG);
   }

/* Setten en resetten van de benodigde hulpfunctie die aangeeft
   of een bus in het traject aanwezig is waarvoor een ingreep
   gewenst is.                                                       */
   if ( inst_EGGPARM(aant_beh_bus_SG) > 0 && !TIJD(uitst_bus_SG) && !HF(bus_op_SG) )
   {    set_HF(bus_op_SG);
   }
   if ( inst_EGGPARM(aant_beh_bus_SG) == 0 && HF(bus_op_SG) )
   {     reset_HF(bus_op_SG);
   }

} /* einde bus_ingreep1 */


void bus_ingreep2 (Int16 n,               /* SG waar bus op rijdt                                  */
                   Int16 in1,             /* inmeldlus 1                                           */
                   Int16 in2,             /* inmeldlus 2                                           */
                   Int16 uit1,            /* uitmeldlus 1                                          */
                   Int16 uit2,            /* uitmeldlus 2                                          */
                   Int16 uitst_bus_SG,    /* tijd uitstel busmelding voor SGn                      */
                   Int16 rijt_bus_SG,     /* ongehinderde rijtijd bus voor SGn                     */
                   Int16 max_groen_SG,    /* maximaal toelaatbare groentijd SGn bij busingreep     */
                   Int16 inm_geb_bus_SG,  /* eparm tbv bepalen inmeldgebied te behandelen bussen   */
                   Int16 aant_bus_SG,     /* aantal ingemelde bussen voor SGn                      */
                   Int16 aant_beh_bus_SG, /* aantal te behandelen bussen voor SGn                  */
                   Int16 bus_op_SG)       /* hulpfunctie te behandelen bus op SGn aanwezig         */
{
/* Inmelding van een bus is niet mogelijk als beide uitmeldlussen storen.
   Omdat soms een van de uitmeldlussen nauwelijks bereden wordt,
   blijft inmelding wel mogelijk als een van de uitmeldlussen stoort.
   Tevens starten van de tijd uitstel busmelding.                    */
   if ( (begin_D(in1) || begin_D(in2)) && !DFOUT(uit1) && !DFOUT(uit2) )
   {    if ( inst_EGGPARM(aant_bus_SG) == 0 )
        {    start_TIJD(uitst_bus_SG);
        }
        wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) + 1);

/* De minimale rijtijd van de bus wordt gestart bij inmelding
   tijdens groen binnen het inmeldgebied                             */
        if ( !G(n) || G(n) && MG_TIJD(n) && (Int16)waarde_MG_TIJD(n) < inst_EPARM(inm_geb_bus_SG) )
        {    herstart_TIJD(rijt_bus_SG);
             wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_beh_bus_SG) + 1);
        }

/* De maximaal toelaatbare groentijd wordt gestart bij begin groen
   of bij inmelding tijdens groen binnen het inmeldgebied            */
        if ( G(n) && MG_TIJD(n) && (Int16)waarde_MG_TIJD(n) < inst_EPARM(inm_geb_bus_SG) )
        {    herstart_TIJD(max_groen_SG);
        }
   }

   if ( begin_G(n) )
   {    herstart_TIJD(max_groen_SG);
   }

/* Op einde groen wordt het aantal te behandelen bussen gelijk
   gesteld aan het aantal ingemelde bussen                           */
   if ( einde_G(n) )
   {    wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_bus_SG) );
   }

/* Uitmelding van een bus en tellen van het aantal buspassages       */
   if ( begin_D(uit1) || begin_D(uit2) )
   {    if ( inst_EGGPARM(aant_bus_SG) > 0 )
        {    wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) - 1);
        }
        if ( inst_EGGPARM(aant_beh_bus_SG) > 0 )
        {    wijzig_EGGPARM(aant_beh_bus_SG, inst_EGGPARM(aant_beh_bus_SG) - 1);
        }
   }

/* Indien de bus voldoende gelegenheid heeft gehad om de stopstreep
   te passeren, wordt de teller die het aantal te behandelen bussen
   aangeeft gereset. De teller van het totaal aantal bussen in het
   traject wordt met het aantal nog te behandelen bussen verlaagd    */
   if (  G(n) && inst_EGGPARM(aant_beh_bus_SG) > 0 && !TIJD(rijt_bus_SG) &&
        (!TIJD(max_groen_SG) || !H1e_TIJD(n) && !H2e_TIJD(n)) )
   {    if ( inst_EGGPARM(aant_bus_SG) >= inst_EGGPARM(aant_beh_bus_SG))
        {    wijzig_EGGPARM(aant_bus_SG, inst_EGGPARM(aant_bus_SG) - inst_EGGPARM(aant_beh_bus_SG));
        }
        wijzig_EGGPARM(aant_beh_bus_SG, 0);
   }

/* Indien het aantal bussen 0 is, worden de daaraan
   gerelateerde tijden gereset                                       */
   if ( inst_EGGPARM(aant_bus_SG) == 0 && TIJD(max_groen_SG) )
   {    reset_TIJD(max_groen_SG);
   }
   if ( inst_EGGPARM(aant_bus_SG) == 0 && TIJD(rijt_bus_SG) )
   {    reset_TIJD(rijt_bus_SG);
   }

/* Setten en resetten van de benodigde hulpfunctie die aangeeft
   of een bus in het traject aanwezig is waarvoor een ingreep
   gewenst is.                                                       */
   if ( inst_EGGPARM(aant_beh_bus_SG) > 0 && !TIJD(uitst_bus_SG) && !HF(bus_op_SG) )
   {    set_HF(bus_op_SG);
   }
   if ( inst_EGGPARM(aant_beh_bus_SG) == 0 && HF(bus_op_SG) )
   {    reset_HF(bus_op_SG);
   }

} /* einde bus_ingreep2 */


void beperk_ingreep (Int16 aant_br,     /* aantal bijzondere realisaties                      */
                     Int16 max_br,      /* maximaal aantal bijzondere realisaties in 1 cyclus */
                     Int16 beperk_bus)  /* beperk ingreep bus gedurende 1 cyclus              */
{

/* Van alle signaalgroepen wordt bijgehouden of er een bijzondere 
   realisatie heeft plaatsgevonden. Indien binnen n cyclus het
   totaal aantal bijzondere realisaties groter wordt dan een
   ingesteld maximum (max_br) wordt gedurende n cyclus de
   busingreep beperkt.                                               */

  Int16 n;

/* Bijhouden aantal bijzondere realisaties                           */
  
  for (n=0; n<NUMSG; n++)
  {    if ( einde_G(n) && BR(n) )
       {    wijzig_EGGPARM(aant_br, inst_EGGPARM(aant_br) + 1);
       }
  }

/* Inmelding van de eerste bus                                       */
  if ( inst_EGGPARM(aant_br) > 0 && 
       inst_EGGPARM(aant_br) < (Uint16) (inst_EGGPARM(max_br)) )
  {    br1e = TRUE;

/* Bijhouden van blokovergangen na de eerste bus.                    */
       if ( br1e && einde_BLOK(blk_m) )
       {    blk1e++;
       }
  }

/* Na n cyclus wordt het aantal bijzondere realisaties op 0 gezet. */
  if ( blk1e > NUMBK )
  {    br1e = FALSE;
       blk1e = 0;
       wijzig_EGGPARM(aant_br, 0);
  }

/* Indien binnen n cyclus het
   totaal aantal bijzondere realisaties groter wordt dan een
   ingesteld maximum wordt gedurende n cyclus de hulpvariabele
   geset die de busingreep beperkt.                                  */
  if ( inst_EGGPARM(aant_br) > 0 &&
       inst_EGGPARM(aant_br) >= (Uint16) (inst_EGGPARM(max_br)) )
  {    br1e = FALSE;
       blk1e = 0;
       set_HF(beperk_bus);
       reset_WB_toewijzing();
  }

/* Bijhouden van blokovergangen na toewijzing beperking ingreep bus. */
  if ( HF(beperk_bus) && einde_BLOK(blk_m) )
  {    blkn++;
  }

/* Na n cyclus wordt het aantal bijzondere realisaties op 0 gezet. */
  if ( blkn > NUMBK )
  {    reset_HF(beperk_bus);
       blkn = 0;
       wijzig_EGGPARM(aant_br, 0);
       set_WB_toewijzing();
  }

}    /* einde  beperk_ingreep  */
