/*
 * Copyright(c)  2002  Rijkswaterstaat
 *                     Dienst Verkeerskunde
 *                     WSRA
 * File:               CRAPTAB.C
 *
 * Nummer kruispunt:        1
 * Regelprogramma vri:      Voorbeeldregeling busingreep
 *
 * datum:                   7 juni 1999
 * gewijzigd:               26 augustus 2004 aanpassing aan RWS C-versie 7.14,
 *                                           gebruik EGGPARMS ipv EPARMS
 *                                           andere afhandeling bussen
 *                                           loskoppeling busingreep van beperking busingreep
 * gewijzigd:               5 oktober 2005   aanpassing PPB_OP in craptab ivm conflicterende BR
 * versie:                  3.2
 */


#include "crtypes.h"
#include "crapcod.h"
#include "craptab.h"
#include "ccie_int.h"
#include "crapdef.h"

/*     BLOKKEN schema                                           */
struct BLOKKEN  BLOKKEN_tabel[] = {
/*     blok:    sg:     toewijzing:                             */
/*     BKl,     SGn,    PRIM + WSGR + ALT,                      */
       BK1,     SG02,   PRIM,

       BK2,     SG05,   PRIM,

       BK3,     SG08,   PRIM,

       BK4,     SG11,   PRIM,
       STOP};

struct WB  WB_tabel[] = { BK1,   STOP};

/*     CONFLICT- en (garantie)ontruimingstijden matrix          */
struct CONFLICT  CONFLICT_tabel[] = {
/*     sg:       conflicterende sg:   GO_TIJD:  O_TIJD:         */
/*     SGn,      SGj,                 x.y,      x.y,            */
       SG02,     SG05,                1.0,      1.0,
       SG02,     SG08,                2.0,      3.0,
       SG02,     SG11,                0.0,      1.0,

       SG05,     SG02,                1.0,      2.0,
       SG05,     SG08,                0.0,      1.0,
       SG05,     SG11,                2.0,      3.0,

       SG08,     SG02,                2.0,      3.0,
       SG08,     SG05,                0.0,      1.0,
       SG08,     SG11,                1.0,      1.0,

       SG11,     SG02,                0.0,      1.0,
       SG11,     SG05,                2.0,      3.0,
       SG11,     SG08,                1.0,      2.0,
       STOP};

/*     SignaalGroep_TIJDEN tabel                                */
struct SG_TIJDEN  SG_TIJDEN_tabel[] = {
/*            instellingen van de tijden:                       */
/*     sg:    GG_TIJD:  VG_TIJD:  GGL_TIJD:  GL_TIJD:  GR_TIJD: */
/*     SGn,    x.y,      x.y,      x.y,       x.y,      x.y,    */
       SG02,   4.0,      8.0,      3.0,       4.0,      3.0,
       SG05,   4.0,      6.0,      3.0,       3.0,      3.0,
       SG08,   4.0,      8.0,      3.0,       4.0,      3.0,
       SG11,   4.0,      6.0,      3.0,       3.0,      3.0,
       STOP};

/*     MG_TIJDEN tabel                                          */
struct MG_TIJDEN  MG_TIJDEN_tabel[] = {
/*     sg:    MG_TIJD:  MG_TIJD1:  MG_TIJD2:  MG_TIJD3:         */
/*     SGn,    x.y,      x.y,       x.y,       x.y,             */
       SG02,  40.0,       NG,       NG,       NG,
       SG05,  30.0,       NG,       NG,       NG,
       SG08,  40.0,       NG,       NG,       NG,
       SG11,  30.0,       NG,       NG,       NG,
       STOP};

/*     HiaatTIJDEN tabel                                        */
struct H_TIJDEN  H_TIJDEN_tabel[] = {
/*               instellingen (x.y of NG) van de tijden:        */
/*     sg:       H1e_TIJD:   H2e_TIJD:                          */
/*     SGn,       x.y,        x.y,                              */
       SG02,      2.0,         NG,
       SG05,      2.0,         NG,
       SG08,      2.0,         NG,
       SG11,      2.0,         NG,
       STOP};

/*     SignaalGroep-DETECTIE tabel                              */
struct SG_DETECTIE           SG_DETECTIE_tabel[] = {
/*     D():     type:        sg:     toewijzing:                */
/*     SGn_i,   DP (of DK),  SGn,    AVR + BS_H5p + H1e + H2e,  */
       SG02_1,  DP,          SG02,   AVR + BS_H5p + H1e,
       SG02_2,  DS,          SG02,   GEEN,
       SG02_3,  DS,          SG02,   GEEN,

       SG05_1,  DP,          SG05,   AVR + BS_H5p + H1e,

       SG08_1,  DP,          SG08,   AVR + BS_H5p + H1e,
       SG08_2,  DS,          SG08,   GEEN,
       SG08_3,  DS,          SG08,   GEEN,
       SG08_4,  DS,          SG08,   GEEN,
       SG08_5,  DS,          SG08,   GEEN,

       SG11_1,  DP,          SG11,   AVR + BS_H5p + H1e,
       STOP};

/*     TIJDENtabel                                              */
struct TIJDEN                TIJDEN_tabel[] = {
/*     TIJD():               instelling:                        */
/*     NAAM,                  x.y,                              */

/**** tijden tbv bus ingreep ******** */
       T_UITST_BUS_02,        2.0, /* uitsteltijd busingreep SG02             */
       T_UITST_BUS_08,        2.0, /* uitsteltijd busingreep SG08             */
       
       T_MIN_BUS_02,         30.0, /* rijtijd bij ongehinderde doorgang SG02  */
       T_MIN_BUS_08,         30.0, /* rijtijd bij ongehinderde doorgang SG08  */

       T_MAX_BUS_02,         60.0, /* maximale groentijd SG02 bij busingreep  */
       T_MAX_BUS_08,         60.0, /* maximale groentijd SG08 bij busingreep  */

       T_AFK_02,             10.0, /* afkappen na x seconden groen SG02       */
       T_AFK_05,              5.0, /* afkappen na x seconden groen SG05       */
       T_AFK_08,             10.0, /* afkappen na x seconden groen SG08       */
       T_AFK_11,              5.0, /* afkappen na x seconden groen SG11       */
       STOP};

/*     EGGPARMStabel                                            */
struct EGGPARMS               EGGPARMS_tabel[] = {
/*     EGGPARM():             instelling:                       */
/*     NAAM,                     x,                             */
       EG_N_BUS_OP_02,           0, /* aantal bussen op SG02                   */
       EG_N_BUS_OP_08,           0, /* aantal bussen op SG08                   */

       EG_N_BEH_BUS_OP_02,       0, /* aantal te behandelen bussen op SG02     */
       EG_N_BEH_BUS_OP_08,       0, /* aantal te behandelen bussen op SG08     */

       EG_BR,                    0, /* aantal bijzondere realisaties           */
       EG_MAX_N_BR,              3, /* maximum aantal bijzondere realisaties   */
       STOP};

/*     EPARMStabel                                              */
struct EPARMS                 EPARMS_tabel[] = {
/*     EPARM():               instelling:                       */
/*     NAAM,                     x,                             */
       EP_INM_GEB_BUS_02,     30.0, /* inmeldgebied tijdens groenfase SG02     */
       EP_INM_GEB_BUS_08,     30.0, /* inmeldgebied tijdens groenfase SG08     */
       STOP};

/*     SG gebonden optie: MA_OP(SGn)                           */

/* Er wordt een aanvraag geset op de richting warop de bus rijdt */
Bool MA_funct_02 (void) { return HF(H_BUS_OP_02) && !GL(SG02) && !GR_TIJD(SG02);}
Bool MA_funct_08 (void) { return HF(H_BUS_OP_08) && !GL(SG08) && !GR_TIJD(SG08);}

struct SG_OPTIE           MA_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
       SG02,              MA_funct_02,
       SG08,              MA_funct_08,
       STOP};

/*     SG gebonden optie: MVG_OP(SGn)                           */
struct SG_OPTIE           MVG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       STOP};

/*     SG gebonden optie: VAA_OP(SGn)                           */
struct SG_OPTIE           VAA_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       SG02,              WAAR,
       SG05,              WAAR,
       SG08,              WAAR,
       SG11,              WAAR,
       STOP};

/*     SG gebonden optie: PPV_OP(SGn)                           */
struct SG_OPTIE           PPV_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               WAAR (of functie),                    */
       SG02,              WAAR,
       SG05,              WAAR,
       SG08,              WAAR,
       SG11,              WAAR,
       STOP};

/*     SG gebonden optie: PPB_OP(SGn)                           */

/* Er wordt een bijzondere realisatie gegeven voor de richting waarop de bus rijdt.
   Dit vindt niet plaats als de busingreep is ingeperkt (bv vanwege te veel
   busingrepen in het verleden) of als er al een bijzondere realisatie is op een
   conflicterende richting. */
Bool PPB_funct_02 (void) {
     return  HF(H_BUS_OP_02) && !PPB_OP(SG08) && !BR(SG08) && !HF(H_BEPERK_BUS); }
Bool PPB_funct_08 (void) {
     return  HF(H_BUS_OP_08) && !PPB_OP(SG02) && !BR(SG02) && !HF(H_BEPERK_BUS); }

struct SG_OPTIE           PPB_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG02,              PPB_funct_02,
       SG08,              PPB_funct_08,
       STOP};

/*     SG gebonden optie: AFK_OP(SGn)                           */

/* De conflicterende richtingen van de richting waarop de bus rijdt wordt afgekapt.
   Als een bijzondere realisatie is gegeven op de richting waarop de bus rijdt,
   wordt deze afgekapt zodra de bus is uitgemeld. */
Bool AFK_funct_02 (void) {
     if begin_G(SG02)                 herstart_TIJD(T_AFK_02);
     return (HF(H_BUS_OP_08) || BR(SG02) && !HF(H_BUS_OP_02)) && !TIJD(T_AFK_02) && !MVG(SG02); } 
Bool AFK_funct_05 (void) {
     if begin_G(SG05)                 herstart_TIJD(T_AFK_05);
     return (HF(H_BUS_OP_02) || HF(H_BUS_OP_08)) && !TIJD(T_AFK_05) && !MVG(SG05); } 
Bool AFK_funct_08 (void) {
     if begin_G(SG08)                 herstart_TIJD(T_AFK_08);
     return (HF(H_BUS_OP_02) || BR(SG08) && !HF(H_BUS_OP_08)) && !TIJD(T_AFK_08) && !MVG(SG08); } 
Bool AFK_funct_11 (void) {
     if begin_G(SG11)                 herstart_TIJD(T_AFK_11);
     return (HF(H_BUS_OP_02) || HF(H_BUS_OP_08)) && !TIJD(T_AFK_11) && !MVG(SG11); } 

struct SG_OPTIE           AFK_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG02,              AFK_funct_02,
       SG05,              AFK_funct_05,
       SG08,              AFK_funct_08,
       SG11,              AFK_funct_11,
       STOP};

/*     SG gebonden optie: RVG_OP(SGn)                           */

/* De conflicterende richtingen van de richting waarop de bus rijdt wordt
   in rood voor groen gehouden */
Bool RVG_funct_02 (void) {
     return BR(SG08); }
Bool RVG_funct_05 (void) {
     return BR(SG02) || BR(SG08); }
Bool RVG_funct_08 (void) {
     return BR(SG02); }
Bool RVG_funct_11 (void) {
     return BR(SG02) || BR(SG08); }

struct SG_OPTIE           RVG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG02,              RVG_funct_02,
       SG05,              RVG_funct_05,
       SG08,              RVG_funct_08,
       SG11,              RVG_funct_11,
       STOP};

/*     SG gebonden optie: VAG_OP(SGn)                           */

/* De richting waarop de bus rijdt wordt vastgehouden in 2eVAG en MVG */
Bool VAG_funct_02 (void) { 
    return HF(H_BUS_OP_02); }
Bool VAG_funct_08 (void) { 
    return HF(H_BUS_OP_08); }

struct SG_OPTIE           VAG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG02,              VAG_funct_02, 
       SG08,              VAG_funct_08, 
       STOP};

/*     SG gebonden optie: VMG_OP(SGn)                           */
struct SG_OPTIE           VMG_OP_tabel[] = {
/*     optie voor sg:     inhoud van de optie:                  */
/*     SGn,               functie,                              */
       SG02,              VAG_funct_02, 
       SG08,              VAG_funct_08, 
       STOP};

/*     REGELINGtabel                                            */
struct REGELING   REGELING_tabel[] = {
/*     Nummer                                                   */
       1,
       STOP};

struct FICT_CONFLICT FICT_CONFLICT_tabel[] = {STOP};
struct SG_BS_H5t SG_BS_H5t_tabel[]  = {STOP};
struct SG_BS_H5ac SG_BS_H5ac_tabel[] = {STOP};
struct SG_BS_H66 SG_BS_H66_tabel[]  = {STOP};
struct E_MG_TIJDEN E_MG_TIJDEN_tabel[] = {STOP};
struct E_H_TIJDEN E_H_TIJDEN_tabel[] = {STOP};
struct SG_RGA SG_RGA_tabel[]  = {STOP};
struct FIETS_VOETG_R FIETS_VOETG_R_tabel[] = {STOP};
struct SG_MAH3 SG_MAH3_tabel[] = {STOP};
struct SG_PPAH SG_PPAH_tabel[] = {STOP};
struct SG_WSRH SG_WSRH_tabel[] = {STOP};
struct KLOKKEN KLOKKEN_tabel[] = {STOP};
struct SCHAKELAAR SCHAKELAAR_tabel[] = {STOP};
struct BK_OPTIE BO_OP_tabel[]  = {STOP};
struct SG_BK_OPTIE PPP_OP_tabel[] = {STOP};
struct SG_OPTIE TR_OP_tabel[]  = {STOP};
struct SG_BK_OPTIE PPA_OP_tabel[] = {STOP};
struct SG_BK_OPTIE VNM_OP_tabel[] = {STOP};
struct SG_OPTIE BL_OP_tabel[]  = {STOP};
struct KOPPEL KOPPEL_tabel[] = {STOP};
